# coding=utf-8

from django.contrib.auth.models import User
from django.test import Client
from django.test import TestCase
from django.urls import reverse

from web.models import MissingSubject
from web.tests.functions import create_worker


class TestRedcapApi(TestCase):
    def setUp(self):
        self.client = Client()
        username = 'piotr'
        password = 'top_secret'
        self.user = User.objects.create_user(
            username=username, email='jacob@bla', password=password)
        self.worker = create_worker(self.user)
        self.client.login(username=username, password=password)

    def test_ignore_missing_subject(self):
        missing_subject = MissingSubject.objects.create(ignore=False)

        response = self.client.get(
            reverse('web.api.redcap.ignore_missing_subject', kwargs={'missing_subject_id': missing_subject.id}))
        self.assertEqual(response.status_code, 200)

        missing_subject = MissingSubject.objects.filter(id=missing_subject.id)[0]

        self.assertTrue(missing_subject.ignore)

    def test_unignore_missing_subject(self):
        missing_subject = MissingSubject.objects.create(ignore=True)

        response = self.client.get(
            reverse('web.api.redcap.unignore_missing_subject', kwargs={'missing_subject_id': missing_subject.id}))
        self.assertEqual(response.status_code, 200)

        missing_subject = MissingSubject.objects.filter(id=missing_subject.id)[0]

        self.assertFalse(missing_subject.ignore)
